@@ -201,7 +201,7 @@ namespace Rcpp{
         private:
             SEXP object;
         };
-        
+
         // Provides only Map::VectorX<t> export
         template<typename T>
         class Exporter<Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, 1> > > {
@@ -223,7 +223,7 @@ namespace Rcpp{
             typedef typename Eigen::Map<Eigen::Matrix<T, 1, Eigen::Dynamic> > OUT ;
             const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
             Rcpp::Vector<RTYPE> vec ;
-          
+
         public:
             Exporter(SEXP x) : vec(x) {
                 if (TYPEOF(x) != RTYPE)
@@ -330,6 +330,26 @@ namespace Rcpp{
                 MatrixExporterForEigen< Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic>, T >(x){}
         };
 
+        // Starting from Eigen 3.3 MappedSparseMatrix was deprecated.
+        // The new type is Map<SparseMatrix>.
+        template<typename T>
+        class Exporter<Eigen::Map<Eigen::SparseMatrix<T> > > {
+        public:
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : d_x(x), d_dims(d_x.slot("Dim")), d_i(d_x.slot("i")), d_p(d_x.slot("p")), xx( d_x.slot("x") ) {
+                if (!d_x.is("dgCMatrix"))
+                    throw std::invalid_argument("Need S4 class dgCMatrix for a mapped sparse matrix");
+            }
+            Eigen::Map<Eigen::SparseMatrix<T> > get() {
+                return Eigen::Map<Eigen::SparseMatrix<T> >(d_dims[0], d_dims[1], d_p[d_dims[1]],
+                                                           d_p.begin(), d_i.begin(), xx.begin() );
+            }
+        protected:
+            S4            d_x;
+            IntegerVector d_dims, d_i, d_p;
+            Vector<RTYPE> xx ;
+        };
+        // Deprecated
         template<typename T>
         class Exporter<Eigen::MappedSparseMatrix<T> > {
         public:
@@ -348,6 +368,26 @@ namespace Rcpp{
             Vector<RTYPE> xx ;
         };
 
+        // Starting from Eigen 3.3 MappedSparseMatrix was deprecated.
+        // The new type is Map<SparseMatrix>.
+        template<typename T>
+        class Exporter<Eigen::Map<Eigen::SparseMatrix<T, Eigen::RowMajor> > > {
+        public:
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : d_x(x), d_dims(d_x.slot("Dim")), d_j(d_x.slot("j")), d_p(d_x.slot("p")), xx( d_x.slot("x") ) {
+                if (!d_x.is("dgRMatrix"))
+                    throw std::invalid_argument("Need S4 class dgRMatrix for a mapped sparse matrix");
+            }
+            Eigen::Map<Eigen::SparseMatrix<T, Eigen::RowMajor> > get() {
+                return Eigen::Map<Eigen::SparseMatrix<T, Eigen::RowMajor> >(d_dims[0], d_dims[1], d_p[d_dims[1]],
+                                                                            d_p.begin(), d_j.begin(), xx.begin() );
+            }
+        protected:
+            S4            d_x;
+            IntegerVector d_dims, d_j, d_p;
+            Vector<RTYPE> xx ;
+        };
+        // Deprecated
         template<typename T>
         class Exporter<Eigen::MappedSparseMatrix<T, Eigen::RowMajor> > {
         public:
